'''Rendering

'''

import pyglet
from pyglet.gl import *
import misc

def initialize():
    '''Create a window, initialize the various pyglet handlers with
routines to handle drawing and resizing the window.

Returns the pyglet window.
'''
    win = pyglet.window.Window(resizable = True, vsync = True,
                               caption = 'Snake RPG')
    win.fps_display = pyglet.clock.ClockDisplay()
    win.g = misc.GenericObject()
    win.g.mouse = misc.Mouse(0, 0)
    win.g.zoom = 1500
    win.g.scroll_x = 0
    win.g.scroll_y = 0
    win.g.nearclip = 100.0
    win.g.farclip = 5000.0
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
    
    @win.event
    def on_resize(width, height):
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(60.0, width / float(height),
                       win.g.nearclip, win.g.farclip)
        glMatrixMode(GL_MODELVIEW)
        if win.g:
            if win.g.gui:
                win.g.gui.resize()
        return pyglet.event.EVENT_HANDLED
    
    @win.event
    def on_draw():
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        if win.g.gamestate == 'playgame':
            draw_game(win)
        elif win.g.gamestate == 'startscreen':
            draw_start_screen(win.g.startscreen)
        elif win.g.gamestate == 'endscreen':
            draw_end_screen(win.g.endscreen)
        elif win.g.gamestate == 'winscreen':
            draw_end_screen(win.g.winscreen)

        glLoadIdentity()
        glTranslatef(win.g.scroll_x, win.g.scroll_y, -win.g.zoom)
        win.g.mouse.draw()

    return win


def draw_start_screen(startscreen):
    '''Draws the title/start screen.  Requires that all the start screen
objects have been properly initialized by calling create_start_screen first.
'''
    glLoadIdentity()
    glTranslatef(0, 0, -700)

    for label in startscreen.labels:
        label.draw()


def draw_end_screen(endscreen):
    glLoadIdentity()
    glTranslatef(0, 0, -700)

    for label in endscreen.labels:
        label.draw()


def draw_game(win):
    '''Draws the active game screen.
    '''
    # Scroll if necessary
    border_x = (win.width - win.g.gui_width) * 0.25
    border_y = win.height * 0.25
    x1, y1 = misc.screen_to_world(win.g.gui_width + border_x, border_y)
    x2, y2 = misc.screen_to_world(win.width - border_x, win.height - border_y)
    #glBegin(GL_LINE_LOOP)
    #glVertex2f(x1, y1)
    #glVertex2f(x2, y1)
    #glVertex2f(x2, y2)
    #glVertex2f(x1, y2)
    #glEnd()
    center_x, center_y = misc.screen_to_world(0, 0)
    edge_x, edge_y =  misc.screen_to_world(win.width - win.g.gui_width,
                                           win.height)
    offset_x = (edge_x - center_x) / 2
    if win.g.player.x < x1:
        offset = x1 - win.g.player.x
        win.g.scroll_x += offset
        win.g.mouse.x -= offset
    if win.g.player.x > x2:
        offset = win.g.player.x - x2
        win.g.scroll_x -= offset
        win.g.mouse.x += offset
    if win.g.player.y < y1:
        offset = y1 - win.g.player.y
        win.g.scroll_y += offset
        win.g.mouse.y -= offset
    if win.g.player.y > y2:
        offset = win.g.player.y - y2
        win.g.scroll_y -= offset
        win.g.mouse.y += offset

    #Start drawing
    glLoadIdentity()
    glTranslatef(win.g.scroll_x, win.g.scroll_y, -win.g.zoom)

    # Draw static geometry
    for sg in win.g.staticgeom:
        sg.draw()

    # Draw exit doors
    for door in win.g.exitlist:
        door.draw()

    # Draw items
    for item in win.g.itemlist:
        item.draw()

    # Draw monsters
    for monster in win.g.monsterlist:
        monster.draw()

    # Draw player
    win.g.player.draw()
    
    # Draw GUI
    win.g.gui.draw()

    #glColor3f(1.0,1.0,0.0)
    #glBegin(GL_LINES)
    #glVertex3f(0.0, 0.0, 0.0)
    #glVertex3f(0.0, 500.0, 0.0)
    #glVertex3f(0.0, 0.0, 0.0)
    #glVertex3f(500.0, 0.0, 0.0)
    #glEnd()

