'''Simulation (most of the game logic)
'''

import pyglet
import level
import combat
import misc

class Sim(object):
    def __init__(self,g):
        self.g = g

    def update(self,dt):
        '''Process one simulation step
        '''
        if self.g.gamestate == 'playgame':
            gamestep(self.g,dt)
        elif self.g.gamestate == 'startscreen':
            self.g.inventory.clear()
            self.g.inventory.score = 0
            self.g.gui.equipitem.clear()
            self.g.curlevel = 0
            self.g.snakelen = 7
            start_screen(self.g)
        elif self.g.gamestate == 'loadlevel':
            # Clean up
            self.g.gui.clearconsole()
            # Load level
            if self.g.curlevel > self.g.maxlevel:
                self.g.gamestate = 'winscreen'
            else:
                level.load_level(self.g, level = self.g.curlevel)
                self.g.gamestate = 'playgame'
                self.g.gui.printconsole('Level %d started!' % self.g.curlevel)
        elif self.g.gamestate == 'endscreen':
            misc.update_score(self.g)
            end_screen(self.g)
        elif self.g.gamestate == 'winscreen':
            misc.update_score(self.g)
            end_screen(self.g)
        else:
            print "CRITICAL ERROR!  Unknown state ",self.g.gamestate
            exit


def start_screen(g):
    '''Logic for start screen
    '''
    if g.inputq.events_waiting():
        ev = g.inputq.get_event()
        if ev[0] == 'key' and ev[1] == pyglet.window.key.M:
            g.sound.toggle_music()
        elif ev[0] == 'key' and ev[1] == pyglet.window.key.F:
            g.showfps = 0
        else:
            g.gamestate = 'loadlevel'
            g.inputq.flush()


def end_screen(g):
    if g.inputq.events_waiting():
        g.gamestate = 'startscreen'
        g.inputq.flush()


def process_events(g, dt):
    for ev in g.inputq:
        if ev[0] == 'key':
            if ev[1] == pyglet.window.key.Q:
                # This sim step will still finish
                g.gamestate = 'startscreen'
            elif ev[1] >= pyglet.window.key._1 or \
                 ev[1] <= pyglet.window.key._5:
                g.inventory.use_item(1 + ev[1] - pyglet.window.key._1)
        elif ev[0] == 'mouseclick':
            if ev[3] & pyglet.window.mouse.RIGHT:
                g.inventory.use_item(1)
            


def gamestep(g, dt):
    # do player movement and physics first for more consistent timing
    # reset forces on player and mobs
    g.player.reset_forces()
    # player movement
    # XXX after adding UI change this to check mouse isn't in UI box
    if (g.mouse.button & pyglet.window.mouse.LEFT):
        g.player.move_to(g.mouse.x, g.mouse.y)
    # Step physics simulation
    g.space.step(g.update_interval)
    # Post-physics updates
    g.player.sync_pos()
    for item in g.itemlist:
        item.post_phystep()
    for monster in g.monsterlist:
        if monster.combat.damage_points <= 0:
            g.gui.printconsole("%s dies!" % monster.combat.name)
            g.sound.play_sound('monster_death')
            monster.die_action()
            monster.state = misc.STATE_REAP
        monster.post_phystep()
    g.arena.update_combat_cooldown(dt)
    # handle other events last as they may take variable time to complete
    process_events(g, dt)
    # check if player died, change game state
    if g.player.combat.damage_points <= 15:
        g.sound.play_same_sound('low_health')
    if g.player.combat.damage_points <= 0:
        g.gamestate = 'endscreen'
        


            


