'''snake mob
'''

import pyglet
import pymunk
import math
import misc
import mob

class Snake(mob.Mob):
    def __init__(self, g, x = 0, y = 0, length = 2, segment_mass = 0.1):
        self.spritelist = []
        self.bodylist = []
        img = pyglet.resource.image('snakehead.png')
        img.anchor_x = img.width // 2
        img.anchor_y = img.height // 2
        self.spritelist.append(pyglet.sprite.Sprite(img, x = x, y = y))
        # Assume this is a circular sprite with width == height
        radius = img.width // 2
        moment = pymunk.moment_for_circle(segment_mass, 0, radius, (0,0))
        body = pymunk.Body(segment_mass, moment)
        body.position.x = x
        body.position.y = y
        g.space.add(body)
        self.bodylist.append(body)
        self.shape = pymunk.Circle(body, radius, (0,0))
        # XXX fix this or no snakes can collide with each other
        self.shape.group = 1
        self.shape.collision_type = misc.COLLTYPE_PLAYER
        g.space.add(self.shape)
        img = pyglet.resource.image('snakebody.png')
        img.anchor_x = img.width // 2
        img.anchor_y = img.height // 2
        offset_step = img.height // 2
        y_offset = y - offset_step
        radius = img.width // 2
        moment = pymunk.moment_for_circle(segment_mass, 0, radius, (0,0))
        # i is index of previous body, as well as counter
        for i in range(length):
            self.spritelist.append(pyglet.sprite.Sprite(img, x = x,
                                                        y = y_offset))
            body = pymunk.Body(segment_mass, moment)
            body.position.x = x
            body.position.y = y_offset
            g.space.add(body)
            self.bodylist.append(body)
            shape = pymunk.Circle(body, radius, (0,0))
            shape.group = 1
            g.space.add(shape)
            # Add joint to previous segment
            prev_body = self.bodylist[i]
            pivot_y = body.position.y + \
                      (prev_body.position.y - body.position.y) / 2
            g.space.add(pymunk.PivotJoint(prev_body, body, (x, pivot_y)))
            y_offset -= offset_step
        self.x = x
        self.y = y
        self.g = g
        self.length = length
        self.segment_mass = segment_mass
        self.acc_per_seg = 10.0
        self.acc = self.acc_per_seg * length
        self.max_force_per_seg = 9000.0 * segment_mass
        self.max_force = self.max_force_per_seg * length
        self.max_force_sqrd = self.max_force ** 2
        # don't try to move to points that are too close
        self.min_movetodist_sqrd = (radius * 2) ** 2

    def __setattr__(self, name, value):
        '''Only move head, let joints deal with rest of segments'''
        object.__setattr__(self, name, value)
        if name == 'x':
            self.spritelist[0].x = value
            self.bodylist[0].position.x = value
        if name == 'y':
            self.spritelist[0].y = value
            self.bodylist[0].position.y = value

    def sync_pos(self):
        object.__setattr__(self, 'x', self.bodylist[0].position.x)
        object.__setattr__(self, 'y', self.bodylist[0].position.y)
        for i in range(len(self.bodylist)):
            self.spritelist[i].x = self.bodylist[i].position.x
            self.spritelist[i].y = self.bodylist[i].position.y

    def move_to(self, x, y):
        v = pymunk.Vec2d(x - self.x, y - self.y)
        f = v * self.acc
        if f.get_length_sqrd() > self.max_force_sqrd:
            f = f.normalized() * self.max_force
        if v.get_length_sqrd() > self.min_movetodist_sqrd:
            self.bodylist[0].apply_force(f, (0,0))
        # Rotate head to look at target position
        self.spritelist[0].rotation = 90 - math.degrees( \
            math.atan2(y - self.y, x - self.x))

    def reset_forces(self):
        # Should only need to reset force on head
        self.bodylist[0].reset_forces()

    def draw(self):
        # Draw sprites from tail to head
        for i in range(len(self.spritelist) - 1, -1, -1):
            self.spritelist[i].draw()
