'''Handle mouse and keyboard input
'''

import misc
from pyglet.gl import *
import event

def initialize(win):
    '''Set up pyglet mouse and keyboard handlers
    '''
    @win.event
    def on_mouse_motion(x, y, dx, dy):
        win.g.mouse.x, win.g.mouse.y = misc.screen_to_world(x, y)

    @win.event
    def on_mouse_scroll(x, y, scroll_x, scroll_y):
        zoom_size = 50
        zoom_max = 4900
        zoom_min = 300
        win.g.zoom -= scroll_y * zoom_size
        if win.g.zoom < zoom_min:
            win.g.zoom = zoom_min
        if win.g.zoom > zoom_max:
            win.g.zoom = zoom_max
        # Scaling changes require mouse position to be recalculated, which
        # requires a "fake" camera repositioning first
        glLoadIdentity()
        glTranslatef(win.g.scroll_x, win.g.scroll_y, -win.g.zoom)
        win.g.mouse.x, win.g.mouse.y = misc.screen_to_world(x, y)

    @win.event
    def on_mouse_press(x, y, button, modifiers):
        win.g.mouse.button |= button
        game_x, game_y = misc.screen_to_world(x, y)
        win.g.inputq.add_mouse_click(game_x, game_y, button, modifiers)

    @win.event
    def on_mouse_drag(x, y, dx, dy, button, modifiers):
        win.g.mouse.x, win.g.mouse.y = misc.screen_to_world(x, y)
        # uncomment this if we end up needing drag events
        # game_x, game_y = misc.screen_to_world(x, y)
        # win.g.inputq.add_mouse_drag(game_x, game_y, button, modifiers)

    @win.event
    def on_mouse_release(x, y, button, modifiers):
        win.g.mouse.button &= ~button

    @win.event
    def on_key_press(symbol, modifiers):
        win.g.inputq.add_keypress(symbol, modifiers)

